@@ -1,6 +1,6 @@
-/* $Id: dialogs.c 17839 2005-09-26 08:35:08Z jasper $ */
-/*-
- * Copyright (c) 2003 Jasper Huijsmans <huysmans@users.sourceforge.net>
+/* $Id: dialogs.c 25769 2007-05-30 02:25:55Z kelnos $ */
+/*
+ * Copyright (c) 2003,2005 Jasper Huijsmans <jasper@xfce.org>
  * Copyright (c) 2003 Benedikt Meurer <benny@xfce.org>
  *
  * This library is free software; you can redistribute it and/or
@@ -467,97 +467,24 @@ xfce_confirm (const gchar *text,
         return FALSE;
 }
 
-/**
- * xfce_message_dialog:
- * @parent           : Transient parent of the dialog, or %NULL
- * @title             : Title of the dialog, or %NULL
- * @icon_id           : Gtk stock icon to show in the dialog
- * @primary_text      : Text shown in large bold font
- * @secondary_text    : Text shown in normal font
- * @first_button_type : Type of the first button, or %NULL
- * @Varargs           : %NULL ended list of parameters and response ID
- *
- * xfce_message_dialog() allows you to easily create a dialog respecting 
- * GNOME HIG; it accepts gtk stock buttons, or custom buttons using a 
- * gtk stock icon and text or a gdkpixbuf and text or just a text. 
- * 
- * It runs a modal message dialog with transient parent @parent (or %NULL 
- * for none) and title @title (or %NULL for a default title) containing 
- * the icon @icon, large and bold @primary_text, normal @secondary_text and 
- * some buttons. 
- *
- * The buttons are defined by first_button_type and the next arguments in
- * following format @type, @param1[, @param2, @param3]; there are four types 
- * of button :
- * <itemizedlist>
- *  <listitem>
- *  <para>
- *  %GTK_STOCK_* : @param1 is used for the response ID, others aren't used.
- *  </para>
- *  </listitem>
- *  <listitem>
- *  <para>
- *  %XFCE_CUSTOM_BUTTON : @param1 is used for the button text and @param2 for 
- *  the the response ID, @param3 isn't used.
- *  </para>
- *  </listitem>
- *  <listitem>
- *  <para>
- *  %XFCE_CUSTOM_STOCK_BUTTON : @param1 is used for the button text, @param2 
- *  for the gtk stock icon name and @param3 for the response ID.
- *  </para>
- *  </listitem>
- *  <listitem>
- *  <para>
- *  %XFCE_CUSTOM_PIXBUF_BUTTON : @param1 is used for the button text, @param2 
- *  for the GdkPixbuf pointer and @param3 for the response ID.
- *  </para>
- *  </listitem>
- * </itemizedlist>  
- * Here's a simple example:
- * <informalexample><programlisting>
- *  gint response = xfce_message_dialog (parent, "Question",
- *                                       GTK_STOCK_DIALOG_QUESTION,
- *                                       "There are unsaved modifications",
- *                                       "The menu has been modified, do you want to save it before quitting?",
- *                                       XFCE_CUSTOM_STOCK_BUTTON, "Forget modifications", 
- *                                       GTK_STOCK_QUIT, GTK_RESPONSE_NO,
- *                                       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
- *                                       GTK_STOCK_SAVE, GTK_RESPONSE_YES, 
- *                                       NULL);
- * </programlisting></informalexample>
- *
- * Return value: selected response ID.
- **/
-gint
-xfce_message_dialog (GtkWindow * parent,
-                     const gchar * title,
-                     const gchar * icon_id,
-                     const gchar * primary_text,
-                     const gchar * secondary_text,
-                     const gchar * first_button_type, ...)
+static GtkWidget *
+xfce_message_dialog_new_valist(GtkWindow *parent,
+                               const gchar *title,
+                               const gchar *icon_id,
+                               const gchar *primary_text,
+                               const gchar *secondary_text,
+                               const gchar *first_button_type,
+                               va_list args)
 {
     GtkWidget *dialog, *label;
     GtkWidget *hbox, *align;
     GtkWidget *image;
     gchar *markup = NULL;
-    gint response = GTK_RESPONSE_NONE;
-    va_list args;
+    gint default_response = G_MININT;
 
     /* create the dialog */
     dialog = gtk_dialog_new ();
 
-    if (parent)
-    {
-        gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
-        gtk_window_set_position (GTK_WINDOW (dialog),
-                                 GTK_WIN_POS_CENTER_ON_PARENT);
-    }
-    else
-    {
-        xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (dialog));
-    }
-
     gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
     gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
     gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
@@ -629,8 +556,6 @@ xfce_message_dialog (GtkWindow * parent,
         const gchar *text;
         gint response_id;
 
-        va_start (args, first_button_type);
-
         text = first_button_type;
 
         while (text != NULL)
@@ -645,7 +570,7 @@ xfce_message_dialog (GtkWindow * parent,
 
                 text = va_arg (args, gchar *);
                 icon = va_arg (args, GdkPixbuf *);
-                response_id = va_arg (args, int);
+                default_response = response_id = va_arg (args, int);
 
                 button = gtk_button_new ();
                 label = gtk_label_new_with_mnemonic (text);
@@ -681,42 +606,180 @@ xfce_message_dialog (GtkWindow * parent,
 
                 text = va_arg (args, gchar *);
                 icon_stock = va_arg (args, gchar *);
-                response_id = va_arg (args, int);
+                default_response = response_id = va_arg (args, int);
 
                 button = xfce_create_mixed_button (icon_stock, text);
             }
             else if (strcmp (text, XFCE_CUSTOM_BUTTON) == 0)
             {
                 text = va_arg (args, gchar *);
-                response_id = va_arg (args, int);
+                default_response = response_id = va_arg (args, int);
 
                 button = gtk_button_new_with_label (text);
             }
             else
             {
-                response_id = va_arg (args, int);
+                default_response = response_id = va_arg (args, int);
 
                 button = gtk_button_new_from_stock (text);
             }
 
+            GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
             gtk_widget_show (button);
             gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button,
                                           response_id);
 
             text = va_arg (args, gchar *);
         }
-
-        va_end (args);
     }
+    
+    if (default_response != G_MININT)
+        gtk_dialog_set_default_response (GTK_DIALOG (dialog), default_response);
 
+    if (parent)
+    {
+        gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
+        gtk_window_set_position (GTK_WINDOW (dialog),
+                                 GTK_WIN_POS_CENTER_ON_PARENT);
+    }
+    else
+    {
+        xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (dialog));
+    }
+    
+    return dialog;
+}
+
+/**
+ * xfce_message_dialog:
+ * @parent            : Transient parent of the dialog, or %NULL
+ * @title             : Title of the dialog, or %NULL
+ * @icon_id           : Gtk stock icon to show in the dialog
+ * @primary_text      : Text shown in large bold font
+ * @secondary_text    : Text shown in normal font
+ * @first_button_type : Type of the first button, or %NULL
+ * @Varargs           : %NULL ended list of parameters and response ID
+ *
+ * xfce_message_dialog() creates a dialog widget as in
+ * xfce_message_dialog_new(), and then runs the dialog as a modal dialog and
+ * returns the response id selected by the user.
+ *
+ * See xfce_message_dialog_new() for more information.
+ *
+ * Returns: The selected response id.
+ **/
+gint
+xfce_message_dialog(GtkWindow *parent,
+                    const gchar *title,
+                    const gchar *icon_id,
+                    const gchar *primary_text,
+                    const gchar *secondary_text,
+                    const gchar *first_button_type,
+                    ...)
+{
+    GtkWidget *dlg;
+    va_list args;
+    gint response;
+    
+    va_start(args, first_button_type);
+    
+    dlg = xfce_message_dialog_new_valist(parent, title, icon_id,
+                                         primary_text, secondary_text,
+                                         first_button_type, args);
+    
+    va_end(args);
+    
     /* show dialog and return */
-    response = gtk_dialog_run (GTK_DIALOG (dialog));
-    gtk_widget_hide (dialog);
-    gtk_widget_destroy (dialog);
+    response = gtk_dialog_run(GTK_DIALOG(dlg));
+    gtk_widget_hide(dlg);
+    gtk_widget_destroy(dlg);
 
     return response;
 }
 
+/**
+ * xfce_message_dialog_new:
+ * @parent            : Transient parent of the dialog, or %NULL
+ * @title             : Title of the dialog, or %NULL
+ * @icon_id           : Gtk stock icon to show in the dialog
+ * @primary_text      : Text shown in large bold font
+ * @secondary_text    : Text shown in normal font
+ * @first_button_type : Type of the first button, or %NULL
+ * @Varargs           : %NULL ended list of parameters and response ID
+ *
+ * xfce_message_dialog_new() allows you to easily create a dialog respecting 
+ * the GNOME HIG; it accepts gtk stock buttons, or custom buttons using a 
+ * gtk stock icon and text or a #GdkPixbuf and text, or just text. 
+ * 
+ * It returns a message dialog with transient parent @parent (or %NULL 
+ * for none) and title @title (or %NULL for a default title) containing 
+ * the icon @icon, large and bold @primary_text, normal @secondary_text and 
+ * some buttons. 
+ *
+ * The buttons are defined by first_button_type and the next arguments in
+ * following format @type, @param1[, @param2, @param3]; there are four types 
+ * of button :
+ * <itemizedlist>
+ *  <listitem>
+ *  <para>
+ *  %GTK_STOCK_* : @param1 is used for the response ID, others aren't used.
+ *  </para>
+ *  </listitem>
+ *  <listitem>
+ *  <para>
+ *  %XFCE_CUSTOM_BUTTON : @param1 is used for the button text and @param2 for 
+ *  the the response ID, @param3 isn't used.
+ *  </para>
+ *  </listitem>
+ *  <listitem>
+ *  <para>
+ *  %XFCE_CUSTOM_STOCK_BUTTON : @param1 is used for the button text, @param2 
+ *  for the gtk stock icon name and @param3 for the response ID.
+ *  </para>
+ *  </listitem>
+ *  <listitem>
+ *  <para>
+ *  %XFCE_CUSTOM_PIXBUF_BUTTON : @param1 is used for the button text, @param2 
+ *  for the GdkPixbuf pointer and @param3 for the response ID.
+ *  </para>
+ *  </listitem>
+ * </itemizedlist>  
+ * Here's a simple example:
+ * <informalexample><programlisting>
+ *  GtkWidget *dlg = xfce_message_dialog (parent, "Question",
+ *                                        GTK_STOCK_DIALOG_QUESTION,
+ *                                        "There are unsaved modifications",
+ *                                        "The menu has been modified, do you want to save it before quitting?",
+ *                                        XFCE_CUSTOM_STOCK_BUTTON, "Forget modifications", 
+ *                                        GTK_STOCK_QUIT, GTK_RESPONSE_NO,
+ *                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+ *                                        GTK_STOCK_SAVE, GTK_RESPONSE_YES, 
+ *                                        NULL);
+ * </programlisting></informalexample>
+ *
+ * Return value: A new #GtkDialog.
+ **/
+GtkWidget *
+xfce_message_dialog_new(GtkWindow *parent,
+                        const gchar *title,
+                        const gchar *icon_id,
+                        const gchar *primary_text,
+                        const gchar *secondary_text,
+                        const gchar *first_button_type,
+                        ...)
+{
+    GtkWidget *dlg;
+    va_list args;
+    
+    va_start(args, first_button_type);
+    dlg = xfce_message_dialog_new_valist(parent, title, icon_id,
+                                         primary_text, secondary_text,
+                                         first_button_type, args);
+    va_end(args);
+    
+    return dlg;
+}
+
 /* command execution with error dialog */
 
 gboolean
