@@ -50,6 +50,8 @@
 #include "netk-xutils.h"
 #include "netk-private.h"
 
+#define NETK_SCREEN_GET_PRIVATE(o)    (G_TYPE_INSTANCE_GET_PRIVATE ((o), NETK_TYPE_SCREEN, NetkScreenPrivate))
+
 static NetkScreen **screens = NULL;
 
 struct _NetkScreenPrivate
@@ -143,50 +145,40 @@ static void emit_background_changed (NetkScreen * screen);
 static void emit_showing_desktop_changed (NetkScreen * screen);
 static void emit_viewports_changed (NetkScreen * screen);
 
-static gpointer parent_class;
+static GObjectClass *parent_class;
 static guint signals[LAST_SIGNAL] = { 0 };
 
 GType
 netk_screen_get_type (void)
 {
-    static GType object_type = 0;
+    static GType type = G_TYPE_INVALID;
 
-    g_type_init ();
-
-    if (!object_type)
+    if (G_UNLIKELY (type == G_TYPE_INVALID))
     {
-        static const GTypeInfo object_info = {
-            sizeof (NetkScreenClass),
-            (GBaseInitFunc) NULL,
-            (GBaseFinalizeFunc) NULL,
-            (GClassInitFunc) netk_screen_class_init,
-            NULL,               /* class_finalize */
-            NULL,               /* class_data */
-            sizeof (NetkScreen),
-            0,                  /* n_preallocs */
-            (GInstanceInitFunc) netk_screen_init,
-            NULL		/* value_table */
-        };
-
-        object_type =
-            g_type_register_static (G_TYPE_OBJECT, "NetkScreen", &object_info,
-                                    0);
+        type = _netk_g_type_register_simple (G_TYPE_OBJECT,
+                                             "NetkScreen",
+                                             sizeof (NetkScreenClass),
+                                             netk_screen_class_init,
+                                             sizeof (NetkScreen),
+                                             netk_screen_init,
+                                             0);                                    
     }
 
-    return object_type;
+    return type;
 }
 
 static void
 netk_screen_init (NetkScreen * screen)
 {
-    screen->priv = g_new0 (NetkScreenPrivate, 1);
-
+    screen->priv = NETK_SCREEN_GET_PRIVATE (screen);
 }
 
 static void
 netk_screen_class_init (NetkScreenClass * klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
+    
+    g_type_class_add_private (klass, sizeof (NetkScreenPrivate));
 
     p_netk_init ();
 
@@ -195,70 +187,70 @@ netk_screen_class_init (NetkScreenClass * klass)
     object_class->finalize = netk_screen_finalize;
 
     signals[ACTIVE_WINDOW_CHANGED] =
-        g_signal_new ("active_window_changed",
+        g_signal_new (I_("active_window_changed"),
                       G_OBJECT_CLASS_TYPE (object_class), G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (NetkScreenClass,
                                        active_window_changed), NULL, NULL,
                       g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
     signals[ACTIVE_WORKSPACE_CHANGED] =
-        g_signal_new ("active_workspace_changed",
+        g_signal_new (I_("active_workspace_changed"),
                       G_OBJECT_CLASS_TYPE (object_class), G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (NetkScreenClass,
                                        active_workspace_changed), NULL, NULL,
                       g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
     signals[WINDOW_STACKING_CHANGED] =
-        g_signal_new ("window_stacking_changed",
+        g_signal_new (I_("window_stacking_changed"),
                       G_OBJECT_CLASS_TYPE (object_class), G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (NetkScreenClass,
                                        window_stacking_changed), NULL, NULL,
                       g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
     signals[WINDOW_OPENED] =
-        g_signal_new ("window_opened", G_OBJECT_CLASS_TYPE (object_class),
+        g_signal_new (I_("window_opened"), G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (NetkScreenClass,
                                                           window_opened),
                       NULL, NULL, g_cclosure_marshal_VOID__OBJECT,
                       G_TYPE_NONE, 1, NETK_TYPE_WINDOW);
 
     signals[WINDOW_CLOSED] =
-        g_signal_new ("window_closed", G_OBJECT_CLASS_TYPE (object_class),
+        g_signal_new (I_("window_closed"), G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (NetkScreenClass,
                                                           window_closed),
                       NULL, NULL, g_cclosure_marshal_VOID__OBJECT,
                       G_TYPE_NONE, 1, NETK_TYPE_WINDOW);
 
     signals[WORKSPACE_CREATED] =
-        g_signal_new ("workspace_created", G_OBJECT_CLASS_TYPE (object_class),
+        g_signal_new (I_("workspace_created"), G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (NetkScreenClass,
                                                           workspace_created),
                       NULL, NULL, g_cclosure_marshal_VOID__OBJECT,
                       G_TYPE_NONE, 1, NETK_TYPE_WORKSPACE);
 
     signals[WORKSPACE_DESTROYED] =
-        g_signal_new ("workspace_destroyed",
+        g_signal_new (I_("workspace_destroyed"),
                       G_OBJECT_CLASS_TYPE (object_class), G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (NetkScreenClass, workspace_destroyed),
                       NULL, NULL, g_cclosure_marshal_VOID__OBJECT,
                       G_TYPE_NONE, 1, NETK_TYPE_WORKSPACE);
 
     signals[APPLICATION_OPENED] =
-        g_signal_new ("application_opened",
+        g_signal_new (I_("application_opened"),
                       G_OBJECT_CLASS_TYPE (object_class), G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (NetkScreenClass, application_opened),
                       NULL, NULL, g_cclosure_marshal_VOID__OBJECT,
                       G_TYPE_NONE, 1, NETK_TYPE_APPLICATION);
 
     signals[APPLICATION_CLOSED] =
-        g_signal_new ("application_closed",
+        g_signal_new (I_("application_closed"),
                       G_OBJECT_CLASS_TYPE (object_class), G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (NetkScreenClass, application_closed),
                       NULL, NULL, g_cclosure_marshal_VOID__OBJECT,
                       G_TYPE_NONE, 1, NETK_TYPE_APPLICATION);
 
     signals[CLASS_GROUP_OPENED] =
-        g_signal_new ("class_group_opened",
+        g_signal_new (I_("class_group_opened"),
                       G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (NetkScreenClass, class_group_opened),
@@ -267,7 +259,7 @@ netk_screen_class_init (NetkScreenClass * klass)
                       G_TYPE_NONE, 1, NETK_TYPE_CLASS_GROUP);
 
     signals[CLASS_GROUP_CLOSED] =
-        g_signal_new ("class_group_closed",
+        g_signal_new (I_("class_group_closed"),
                       G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (NetkScreenClass, class_group_closed),
@@ -276,21 +268,21 @@ netk_screen_class_init (NetkScreenClass * klass)
                       G_TYPE_NONE, 1, NETK_TYPE_CLASS_GROUP);
 
     signals[BACKGROUND_CHANGED] =
-        g_signal_new ("background_changed",
+        g_signal_new (I_("background_changed"),
                       G_OBJECT_CLASS_TYPE (object_class), G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (NetkScreenClass, background_changed),
                       NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
                       0);
 
     signals[SHOWING_DESKTOP_CHANGED] =
-        g_signal_new ("showing_desktop_changed",
+        g_signal_new (I_("showing_desktop_changed"),
                       G_OBJECT_CLASS_TYPE (object_class), G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (NetkScreenClass, showing_desktop_changed),
                       NULL, NULL,
                       g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
     signals[VIEWPORTS_CHANGED] =
-        g_signal_new ("viewports_changed",
+        g_signal_new (I_("viewports_changed"),
                       G_OBJECT_CLASS_TYPE (object_class), G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (NetkScreenClass, viewports_changed),
                       NULL, NULL,
@@ -316,8 +308,6 @@ netk_screen_finalize (GObject * object)
 
     screens[screen->priv->number] = NULL;
 
-    g_free (screen->priv);
-
     G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -1004,6 +994,8 @@ update_client_list (NetkScreen * screen)
     g_list_free (created);
     g_list_free (closed_apps);
     g_list_free (created_apps);
+    g_list_free (closed_class_groups);
+    g_list_free (created_class_groups);
 
     --reentrancy_guard;
 
@@ -1116,7 +1108,7 @@ update_workspace_list (NetkScreen * screen)
     tmp = deleted;
     while (tmp != NULL)
     {
-        g_object_unref (tmp->data);
+        g_object_unref (G_OBJECT (tmp->data));
 
         tmp = tmp->next;
     }
